@@ -104,10 +104,13 @@ static struct sock_opt {
 	SOCK_OPT(SOL_SOCKET, SO_RCVTIMEO, struct timeval)
 
 	SOCK_OPT(IPPROTO_TCP, TCP_NODELAY, int)
-#ifdef HAVE_TCP_KEEP
+
+#if defined(HAVE_TCP_KEEP)
 	SOCK_OPT(IPPROTO_TCP, TCP_KEEPIDLE, int)
 	SOCK_OPT(IPPROTO_TCP, TCP_KEEPCNT, int)
 	SOCK_OPT(IPPROTO_TCP, TCP_KEEPINTVL, int)
+#elif defined(HAVE_TCP_KEEPALIVE)
+	SOCK_OPT(IPPROTO_TCP, TCP_KEEPALIVE, int)
 #endif
 
 #undef SOCK_OPT
@@ -205,13 +208,16 @@ vca_sock_opt_init(void)
 		NEW_VAL(SO_RCVTIMEO, so, tv,
 		    VTIM_timeval(cache_param->timeout_idle));
 		SET_VAL(TCP_NODELAY, so, i, enable_tcp_nodelay);
-#ifdef HAVE_TCP_KEEP
+#if defined(HAVE_TCP_KEEP)
 		NEW_VAL(TCP_KEEPIDLE, so, i,
 		    (int)cache_param->tcp_keepalive_time);
 		NEW_VAL(TCP_KEEPCNT, so, i,
 		    (int)cache_param->tcp_keepalive_probes);
 		NEW_VAL(TCP_KEEPINTVL, so, i,
 		    (int)cache_param->tcp_keepalive_intvl);
+#elif defined(HAVE_TCP_KEEPALIVE)
+		NEW_VAL(TCP_KEEPALIVE, so, i,
+		    (int)cache_param->tcp_keepalive_time);
 #endif
 	}
 	return (chg);
@@ -456,7 +462,7 @@ vca_make_session(struct worker *wrk, void *arg)
 	}
 	vca_sock_opt_set(wa->acceptlsock, sp);
 
-	req = Req_New(wrk, sp);
+	req = Req_New(sp);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	req->htc->rfd = &sp->fd;
 
@@ -780,21 +786,28 @@ VCA_Shutdown(void)
 static VTAILQ_HEAD(,transport)	transports =
     VTAILQ_HEAD_INITIALIZER(transports);
 
+static uint16_t next_xport;
+
+static void
+XPORT_Register(struct transport *xp)
+{
+
+	CHECK_OBJ_NOTNULL(xp, TRANSPORT_MAGIC);
+	AZ(xp->number);
+
+	xp->number = ++next_xport;
+	VTAILQ_INSERT_TAIL(&transports, xp, list);
+}
+
 void
 XPORT_Init(void)
 {
-	uint16_t n;
-	struct transport *xp;
 
 	ASSERT_MGT();
 
-	VTAILQ_INSERT_TAIL(&transports, &PROXY_transport, list);
-	VTAILQ_INSERT_TAIL(&transports, &HTTP1_transport, list);
-	VTAILQ_INSERT_TAIL(&transports, &H2_transport, list);
-
-	n = 0;
-	VTAILQ_FOREACH(xp, &transports, list)
-		xp->number = ++n;
+#define TRANSPORT_MACRO(name) XPORT_Register(&name##_transport);
+	TRANSPORTS
+#undef TRANSPORT_MACRO
 }
 
 const struct transport *
