@@ -76,6 +76,7 @@ static const char * const vsl_r_names[VSL_r__MAX] = {
 };
 
 struct vtx;
+VTAILQ_HEAD(vtxhead, vtx);
 
 struct vslc_raw {
 	unsigned		magic;
@@ -161,12 +162,12 @@ struct vtx {
 	enum VSL_reason_e	reason;
 
 	struct vtx		*parent;
-	VTAILQ_HEAD(,vtx)	child;
+	struct vtxhead		child;
 	unsigned		n_child;
 	unsigned		n_childready;
 	unsigned		n_descend;
 
-	VTAILQ_HEAD(,synth)	synth;
+	struct synthhead	synth;
 
 	struct chunk		shmchunks[VTX_SHMCHUNKS];
 	struct chunkhead	shmchunks_free;
@@ -189,11 +190,11 @@ struct VSLQ {
 
 	/* Structured mode */
 	struct vtx_tree		tree;
-	VTAILQ_HEAD(,vtx)	ready;
-	VTAILQ_HEAD(,vtx)	incomplete;
+	struct vtxhead		ready;
+	struct vtxhead		incomplete;
 	int			n_outstanding;
 	struct chunkhead	shmrefs;
-	VTAILQ_HEAD(,vtx)	cache;
+	struct vtxhead		cache;
 	unsigned		n_cache;
 
 	/* Rate limiting */
@@ -562,7 +563,8 @@ vtx_retire(struct VSLQ *vslq, struct vtx **pvtx)
 	AZ(vtx->n_child);
 	AZ(vtx->n_descend);
 	vtx->n_childready = 0;
-	AN(VRBT_REMOVE(vtx_tree, &vslq->tree, &vtx->key));
+	// remove rval is no way to check if element was present
+	(void)VRBT_REMOVE(vtx_tree, &vslq->tree, &vtx->key);
 	vtx->key.vxid = 0;
 	vtx->flags = 0;
 
@@ -687,10 +689,10 @@ vtx_set_parent(struct vtx *parent, struct vtx *child)
    successfully parsed. */
 static int
 vtx_parse_link(const char *str, enum VSL_transaction_e *ptype,
-    unsigned *pvxid, enum VSL_reason_e *preason)
+    unsigned *pvxid, enum VSL_reason_e *preason, unsigned *psub)
 {
 	char type[16], reason[16];
-	unsigned vxid;
+	unsigned vxid, sub;
 	int i;
 	enum VSL_transaction_e et;
 	enum VSL_reason_e er;
@@ -700,7 +702,7 @@ vtx_parse_link(const char *str, enum VSL_transaction_e *ptype,
 	AN(pvxid);
 	AN(preason);
 
-	i = sscanf(str, "%15s %u %15s", type, &vxid, reason);
+	i = sscanf(str, "%15s %u %15s %u", type, &vxid, reason, &sub);
 	if (i < 1)
 		return (0);
 
@@ -727,7 +729,13 @@ vtx_parse_link(const char *str, enum VSL_transaction_e *ptype,
 	if (er >= VSL_r__MAX)
 		er = VSL_r_unknown;
 	*preason = er;
-	return (3);
+	if (i == 3)
+		return (3);
+
+	/* request sub-level */
+	if (psub != NULL)
+		*psub = sub;
+	return (4);
 }
 
 /* Parse and process a begin record */
@@ -744,8 +752,8 @@ vtx_scan_begin(struct VSLQ *vslq, struct vtx *vtx, const uint32_t *ptr)
 
 	AZ(vtx->flags & VTX_F_READY);
 
-	i = vtx_parse_link(VSL_CDATA(ptr), &type, &p_vxid, &reason);
-	if (i != 3)
+	i = vtx_parse_link(VSL_CDATA(ptr), &type, &p_vxid, &reason, NULL);
+	if (i < 3)
 		return (vtx_diag_tag(vtx, ptr, "parse error"));
 	if (type == VSL_t_unknown)
 		(void)vtx_diag_tag(vtx, ptr, "unknown vxid type");
@@ -811,8 +819,8 @@ vtx_scan_link(struct VSLQ *vslq, struct vtx *vtx, const uint32_t *ptr)
 
 	AZ(vtx->flags & VTX_F_READY);
 
-	i = vtx_parse_link(VSL_CDATA(ptr), &c_type, &c_vxid, &c_reason);
-	if (i != 3)
+	i = vtx_parse_link(VSL_CDATA(ptr), &c_type, &c_vxid, &c_reason, NULL);
+	if (i < 3)
 		return (vtx_diag_tag(vtx, ptr, "parse error"));
 	if (c_type == VSL_t_unknown)
 		(void)vtx_diag_tag(vtx, ptr, "unknown vxid type");
@@ -933,8 +941,7 @@ vslq_ratelimit(struct VSLQ *vslq)
 	now = VTIM_mono();
 	delta = now - vslq->last_use;
 	vslq->credits += (delta / vslq->vsl->R_opt_p) * vslq->vsl->R_opt_l;
-	if (vslq->credits > vslq->vsl->R_opt_l)
-		vslq->credits = vslq->vsl->R_opt_l;
+	vslq->credits = vmin_t(double, vslq->credits, vslq->vsl->R_opt_l);
 	vslq->last_use = now;
 
 	if (vslq->credits < 1.0)
@@ -1287,7 +1294,7 @@ vslq_candidate(struct VSLQ *vslq, const uint32_t *ptr)
 	enum VSL_reason_e reason;
 	struct VSL_data *vsl;
 	enum VSL_tag_e tag;
-	unsigned p_vxid;
+	unsigned p_vxid, sub;
 	int i;
 
 	CHECK_OBJ_NOTNULL(vslq, VSLQ_MAGIC);
@@ -1301,24 +1308,24 @@ vslq_candidate(struct VSLQ *vslq, const uint32_t *ptr)
 	CHECK_OBJ_NOTNULL(vsl, VSL_MAGIC);
 	if (vslq->grouping == VSL_g_vxid) {
 		if (!vsl->c_opt && !vsl->b_opt)
-			return (1); /* Implies also !vsl->E_opt */
-		if (!vsl->b_opt && !VSL_CLIENT(ptr))
+			AZ(vsl->E_opt);
+		else if (!vsl->b_opt && !VSL_CLIENT(ptr))
 			return (0);
-		if (!vsl->c_opt && !VSL_BACKEND(ptr))
+		else if (!vsl->c_opt && !VSL_BACKEND(ptr))
 			return (0);
 		/* Need to parse the Begin tag - fallthrough to below */
 	}
 
 	tag = VSL_TAG(ptr);
 	assert(tag == SLT_Begin);
-	i = vtx_parse_link(VSL_CDATA(ptr), &type, &p_vxid, &reason);
-	if (i != 3 || type == VSL_t_unknown)
+	i = vtx_parse_link(VSL_CDATA(ptr), &type, &p_vxid, &reason, &sub);
+	if (i < 3 || type == VSL_t_unknown)
 		return (0);
 
-	if (type == VSL_t_sess)
+	if (vslq->grouping == VSL_g_request && type == VSL_t_sess)
 		return (0);
 
-	if (vslq->grouping == VSL_g_vxid && reason == VSL_r_esi && !vsl->E_opt)
+	if (vslq->grouping == VSL_g_vxid && i > 3 && sub > 0 && !vsl->E_opt)
 		return (0);
 
 	return (1);
